<?php

namespace FiloBlu\ExtAnowaveGtmEnhancedEcommerceUaTrackingEc\Helper;

use Magento\Store\Model\Store;
use Anowave\Package\Helper\Package;
use Magento\Framework\Registry;
use Anowave\Package\Helper\Base;

class Data extends \Anowave\Ec\Helper\Data {

    /**
     * Get checkout products 
     * 
     * @param unknown $block
     * @param \Magento\Checkout\Model\Cart $cart
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\ObjectManagerInterface $object
     */
    public function getCheckoutProducts($block, \Magento\Checkout\Model\Cart $cart, \Magento\Framework\Registry $registry, \Magento\Framework\ObjectManagerInterface $object) {
        $products = array();


        foreach ($cart->getQuote()->getAllVisibleItems() as $item) {
            /**
             * Get all product categories
             */
            $categories = $item->getProduct()->getCategoryIds();

            /**
             * Load first category 
             */
            $category = $object->create('\Magento\Catalog\Model\Category')->load
                    (
                    reset($categories)
            );

            $variant = [];

            $data = new \Magento\Framework\DataObject(array
                (
                'id' => $item->getSku(),
                'name' => $item->getName(),
                'price' => $item->getPriceInclTax(),
                'quantity' => (int) $item->getQty(),
                'category' => $this->getCategory($category),
                'brand' => $this->getBrand
                        (
                        $item->getProduct()
                )
            ));

            if ('configurable' == $item->getProduct()->getTypeId()) {
                $variant = array();

                /**
                 * Get info buy request
                 *
                 * @var \Magento\Framework\DataObject
                 */
                $buyRequest = $item->getProduct()->getCustomOption('info_buyRequest');

                if (isset($buyRequest['value'])) {
                    $value = unserialize($buyRequest['value']);

                    $info = new \Magento\Framework\DataObject($value);
                } else {
                    $info = new \Magento\Framework\DataObject(array
                            (
                            /**
                             * Check
                             */
                    ));
                }

                /**
                 * Construct variant
                 */
                foreach ((array) $info->getSuperAttribute() as $id => $option) {
                    /**
                     * @todo: Pull attribute data
                     */
                    $attribute = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\ResourceModel\Eav\Attribute')->load($id);

                    if ($attribute->usesSource()) {
                        $name = ($this->useDefaultValues() ? $attribute->getFrontendLabel() : $attribute->getStoreLabel());
                        $text = $attribute->getSource()->getOptionText($option);

                        if ($this->useDefaultValues()) {
                            /**
                             * Get current store
                             * 
                             * @var int
                             */
                            $currentStore = $attribute->getSource()->getAttribute()->getStoreId();

                            /**
                             * Change default store
                             */
                            $attribute->getSource()->getAttribute()->setStoreId(0);

                            /**
                             * Get text
                             * 
                             * @var string
                             */
                            $text = $attribute->getSource()->getOptionText($option);

                            /**
                             * Restore store
                             */
                            $attribute->getSource()->getAttribute()->setStoreId($currentStore);
                        }

                        $variant[] = join(':', array($name, $text));
                    }
                }

                $data->setId
                        (
                        $item->getProduct()->getSku()
                );

                $data->setName
                        (
                        $item->getProduct()->getName()
                );

                /**
                 * Load configurable
                 */
                $configurable = $this->productRepository->getById
                        (
                        $item->getProductId()
                );

                /**
                 * Swap data with configurable
                 */
                $data->setId($configurable->getSku());
                $data->setName($configurable->getName());
                $data->setBrand
                        (
                        $this->getBrand($configurable)
                );

                /**
                 * Push variant to data
                 *
                 * @var array
                 */
                $data->setVariant(join(chr(44), $variant));
            }

            $products[] = $data->getData();
        }

        return $products;
    }

    /**
     * Impressions push 
     * 
     * @param Block $block
     */
    public function getImpressionPushForward($block) {
        try {
            $list = $block->getLayout()->getBlock('category.products.list');

            if ($list) {
                $categoryList = $this->registry->registry('current_category');

                $response = array
                    (
                    'currencyCode' => $this->getStore()->getCurrentCurrencyCode(),
                    'ecommerce' => array
                        (
                        'actionField' => array
                            (
                            'list' => $this->getCategoryList($categoryList)
                        ),
                        'impressions' => array()
                    )
                );

                $position = 1;

                $data = array();

                $categories = array();
                foreach ($this->getLoadedCollection($list) as $product) {

                    $categoriesId = $product->getCategoryIds();

                    /**
                     * Load first category
                     */
                    $catId = reset($categoriesId);
                    if (!isset($categories[$catId])) {
                        $category = \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Category')->load($catId);
                        $categories[$catId] = $category;
                    } else {
                        $category = $categories[$catId];
                    }

                    $response['ecommerce']['impressions'][] = array
                        (
                        'list' => $this->getCategoryList($categoryList),
                        'category' => $this->getCategory($category),
                        'id' => $product->getSku(),
                        'name' => $product->getName(),
                        'brand' => $this->getBrand
                                (
                                $product
                        ),
                        'price' => $this->getPrice($product),
                        'position' => $position++
                    );
                }

                $response['currentStore'] = $this->getStoreName();
            }

            return (object)
                    [
                        'push' => json_encode($response, JSON_PRETTY_PRINT),
                        'google_tag_params' => array
                            (
                            'ecomm_pagetype' => 'category',
                            'ecomm_category' => $this->escape($this->getCategory($categoryList))
                        )
            ];
        } catch (\Exception $e) {
            
        }

        return false;
    }

    /**
     * Get loaded product collection from product list block 
     *  
     * @param \Magento\Catalog\Block\Product\ListProduct $list
     */
    private function getLoadedCollection(\Magento\Catalog\Block\Product\ListProduct $list) {
        $collection = $list->getLoadedProductCollection();

        /**
         * Get toolbar
         */
        $toolbar = $list->getToolbarBlock();

        if ($toolbar) {
            $orders = $list->getAvailableOrders();

            if ($orders) {
                $toolbar->setAvailableOrders($orders);
            }

            $sort = $list->getSortBy();

            if ($sort) {
                $toolbar->setDefaultOrder($sort);
            }

            $dir = $list->getDefaultDirection();

            if ($dir) {
                $toolbar->setDefaultDirection($dir);
            }

            $modes = $list->getModes();

            if ($modes) {
                $toolbar->setModes($modes);
            }

//			$collection->setCurPage($toolbar->getCurrentPage());

            $limit = (int) $toolbar->getLimit();

            if ($limit) {
//                          $collection->setPageSize($limit);
                $_categoryId = -1;

                // for avoid problems on search
                if ($this->registry->registry("current_category")) {
                    $_categoryId = $this->registry->registry("current_category")->getId();
                }

                $offset = 0;
                $page = (int) $this->request->getParam('p', 1);

                if ($page > 1) {

                    $listMode = $toolbar->getCurrentMode();
                    $productNumberPage1 = $this->_helperCategoryPagination->getProductNumber($listMode, 1, $_categoryId);
                    $productNumberPageOther = $this->_helperCategoryPagination->getProductNumber($listMode, 2, $_categoryId);

                    $offset = $productNumberPage1 + ($productNumberPageOther * ($page - 2));

                    if ($this->_helperCategoryPagination->getConfig('general/load_all_products_before') && !$this->request->isXmlHttpRequest()) {
                        $limit = $offset + $limit;
                        $offset = 0;
                    }
                }

                $collection->getSelect()->limit($limit, $offset);
            }

            if ($toolbar->getCurrentOrder()) {
                $collection->setOrder($toolbar->getCurrentOrder(), $toolbar->getCurrentDirection());
            }
        }

        return $collection;
    }

    public function getDetailPushForward($block) {


        $info = $block->getLayout()->getBlock('product.info');

        if ($info) {
            $categoryList = $this->registry->registry('current_category');

            /**
             * Get all product categories
             */
            $categories = $info->getProduct()->getCategoryIds();

            /**
             * Load first category
             */
            $category = \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Category')->load
                    (
                    reset($categories)
            );

            $data = array
                (
                'currencyCode' => $this->getStore()->getCurrentCurrencyCode(),
                'ecommerce' => array
                    (
                    'detail' => array
                        (
                        'actionField' => array
                            (
                            'list' => $this->getCategoryList($categoryList)
                        ),
                        'products' => array
                            (
                            array
                                (
                                'id' => $info->getProduct()->getSku(),
                                'name' => $info->getProduct()->getName(),
                                'price' => $this->getPrice($info->getProduct()),
                                'brand' => $this->getBrand
                                        (
                                        $info->getProduct()
                                ),
                                'category' => $this->getCategory($category),
                                'quantity' => 1
                            )
                        )
                    )
                )
            );

            $data['currentStore'] = $this->getStoreName();

            return (object)
                    [
                        'push' => json_encode($data, JSON_PRETTY_PRINT),
                        'fbq' => json_encode($this->getFacebookViewContentTrack($info->getProduct(), $category)),
                        'google_tag_params' => array
                            (
                            'ecomm_pagetype' => 'product',
                            'ecomm_category' => $this->escape($this->getCategory($category)),
                            'ecomm_prodid' => $this->escape($info->getProduct()->getSku()),
                            'ecomm_totalvalue' => $this->getPrice($info->getProduct())
                        ),
                        'group' => $this->getDetailGroup($info, $category)
            ];
        }

        return false;
    }

    public function getPurchasePush($block) {
        foreach ($this->getOrders($block) as $order) {
            $response = array
                (
                'ecommerce' => array
                    (
                    'currencyCode' => $this->getStore()->getCurrentCurrencyCode(),
                    'purchase' =>
                    [
                        'actionField' =>
                        [
                            'id' => $order->getIncrementId(),
                            'revenue' => $order->getBaseGrandTotal(),
                            'tax' => $order->getBaseTaxAmount(),
                            'shipping' => $order->getBaseShippingAmount(),
                            'coupon' => (string) $order->getCouponCode(),
                            'affiliation' => (string) $this->getStore()->getName()
                        ],
                        'products' => []
                    ]
                ),
                'facebook' =>
                [
                    'revenue' => $order->getBaseGrandTotal(),
                    'subtotal' => $order->getBaseSubtotal()
                ]
            );

            if ($order->getIsVirtual()) {
                $address = $order->getBillingAddress();
            } else {
                $address = $order->getShippingAddress();
            }

            foreach ($order->getAllVisibleItems() as $item) {
                $variant = array();

                /**
                 * Get all product categories
                 */
                $categories = $item->getProduct()->getCategoryIds();

                /**
                 * Load first category
                 */
                $category = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Catalog\Model\Category')->load
                        (
                        reset($categories)
                );

                $data = new \Magento\Framework\DataObject(array
                    (
                    'id' => $item->getSku(),
                    'name' => $item->getName(),
                    'price' => $item->getBasePrice(),
                    'quantity' => (int) $item->getQtyOrdered(),
                    'category' => $this->getCategory($category),
                    'brand' => $this->getBrand
                            (
                            $item->getProduct()
                    )
                ));

                if ('configurable' == $item->getProduct()->getTypeId()) {
                    $variant = array();

                    /**
                     * Get info buy request 
                     * 
                     * @var \Magento\Framework\DataObject
                     */
                    $info = new \Magento\Framework\DataObject($item->getProductOptionByCode('info_buyRequest'));

                    /**
                     * Construct variant
                     */
                    foreach ($info->getSuperAttribute() as $id => $option) {
                        /**
                         * @todo: Pull attribute data
                         */
                        $attribute = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\ResourceModel\Eav\Attribute')->load($id);

                        if ($attribute->usesSource()) {
                            $name = ($this->useDefaultValues() ? $attribute->getFrontendLabel() : $attribute->getStoreLabel());
                            $text = $attribute->getSource()->getOptionText($option);

                            if ($this->useDefaultValues()) {
                                /**
                                 * Get current store
                                 *
                                 * @var int
                                 */
                                $currentStore = $attribute->getSource()->getAttribute()->getStoreId();

                                /**
                                 * Change default store
                                 */
                                $attribute->getSource()->getAttribute()->setStoreId(0);

                                /**
                                 * Get text
                                 *
                                 * @var string
                                 */
                                $text = $attribute->getSource()->getOptionText($option);

                                /**
                                 * Restore store
                                 */
                                $attribute->getSource()->getAttribute()->setStoreId($currentStore);
                            }

                            $variant[] = join(':', array($name, $text));
                        }
                    }

                    $data->setId
                            (
                            $item->getProduct()->getSku()
                    );

                    $data->setName
                            (
                            $item->getProduct()->getName()
                    );

                    $product = $this->productRepository->getById
                            (
                            $item->getProductId()
                    );

                    if (false) {
                        /**
                         * Get parents
                         *
                         * @var array
                         */
                        $parents = (array) \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\ConfigurableProduct\Model\Product\Type\Configurable')->getParentIdsByChild
                                        (
                                        $product->getId()
                        );

                        $configurable = $this->productRepository->getById
                                (
                                end($parents)
                        );
                    }

                    /**
                     * Push variant to data
                     *
                     * @var array
                     */
                    $data->setVariant(join(chr(44), $variant));
                }

                $response['ecommerce']['purchase']['products'][] = $data->getData();
            }

            $response['currentStore'] = $this->getStoreName();
        }

        return json_encode($response, JSON_PRETTY_PRINT);
    }

    public function getSearchPush($block) {
        try {
            $list = $block->getLayout()->getBlock('search_result_list');

            if ($list) {
                $response = array
                    (
                    'currencyCode' => $this->getStore()->getCurrentCurrencyCode(),
                    'ecommerce' => array
                        (
                        'actionField' => array
                            (
                            'list' => 'Search Results'
                        ),
                        'impressions' => array()
                    )
                );

                $position = 1;

                $data = array();

                foreach ($this->getLoadedCollection($list) as $product) {

                    $categories = $product->getCategoryIds();

                    /**
                     * Load first category
                     */
                    $category = \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Category')->load
                            (
                            reset($categories)
                    );

                    $response['ecommerce']['impressions'][] = array
                        (
                        'list' => 'Search Results',
                        'category' => $this->getCategory($category),
                        'id' => $product->getSku(),
                        'name' => $product->getName(),
                        'brand' => $this->getBrand
                                (
                                $product
                        ),
                        'price' => $this->getPrice($product),
                        'position' => $position++
                    );
                }

                $response['currentStore'] = $this->getStoreName();
            }

            return (object)
                    [
                        'push' => json_encode($response, JSON_PRETTY_PRINT),
                        'google_tag_params' => array
                            (
                            'ecomm_pagetype' => 'category',
                            'ecomm_category' => 'Search Results'
                        )
            ];
        } catch (\Exception $e) {
            
        }

        return false;
    }

    /**
     * Get category 
     * 
     * @param \Magento\Catalog\Model\Category $category
     */
    public function getCategory(\Magento\Catalog\Model\Category $category) {
        if (!$category) {
            return '';
        }
        if (0 !== (int) $this->getConfig('ec/options/use_segments')) {
            return $this->getCategorySegments($category);
        }

        $useSeoCategoryNameIExist = $this->scopeConfig->getValue('mageworx_seo/seoxtemplates/use_category_seo_name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->getStore()) && $category->getCategorySeoName();
        if ($useSeoCategoryNameIExist) {
            return $category->getCategorySeoName();
        }

        return $category->getName();
    }

    /**
     * Get category list name
     * 
     * @param \Magento\Catalog\Model\Category $category
     */
    public function getCategoryList(\Magento\Catalog\Model\Category $category) {
        if (!$category) {
            return '';
        }

        $useSeoCategoryNameIExist = $this->scopeConfig->getValue('mageworx_seo/seoxtemplates/use_category_seo_name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->getStore()) && $category->getCategorySeoName();
        if ($useSeoCategoryNameIExist) {
            return $category->getCategorySeoName();
        }

        return $category->getName();
    }

    /**
     * Retrieve category and it's parents separated by chr(47)
     *
     * @param Mage_Catalog_Model_Category $category
     * @return string
     */
    public function getCategorySegments(\Magento\Catalog\Model\Category $category) {
        $segments = array();

        foreach ($category->getParentCategories() as $parent) {
            $useSeoCategoryNameIExist = $this->scopeConfig->getValue('mageworx_seo/seoxtemplates/use_category_seo_name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->getStore()) && $parent->getCategorySeoName();
            if ($useSeoCategoryNameIExist) {
                $segments[] = $parent->getCategorySeoName();
            }
            else{
                $segments[] = $parent->getName();
            }
        }

        if (!$segments) {
            $useSeoCategoryNameIExist = $this->scopeConfig->getValue('mageworx_seo/seoxtemplates/use_category_seo_name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->getStore()) && $category->getCategorySeoName();
            if ($useSeoCategoryNameIExist) {
                $segments[] = $category->getCategorySeoName();
            }
            else{
                $segments[] = $category->getName();
            }
        }

        return trim(join(chr(47), $segments));
    }

}
